<?php
namespace App\Http\Controllers;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Str;    //for UUID
use Illuminate\Support\Facades\Password;


class UserController extends Controller
{
    public function showRegistrationForm()
    {
        return view('frontend.pages.register');
    }
    
    public function registerUser(Request $request)
    {
        $request->validate([
            'username'=>'required',
            'email'=>'required|email|unique:users,email',
            'password'=>'required|min:8|confirmed',
        ]);
        
        $user = new User();
        $user->uid = (string) Str::uuid();  // Generates a unique UUID       
        $user->username = $request->username;
        $user->email = $request->email;
        $user->user_type = 'superadmin';
        $user->password = $request->password;
        $user->created_at = time();
        $user->updated_at = time();

        $result = $user->save();
        if ($user->save()) {
            return response()->json(['status' => 'success', 'message' => 'User registered successfully.'], 201);
        } else {
            return response()->json(['status' => 'error', 'message' => 'Failed to register user.'], 500);
        }
    }

    public function showLoginForm()
    {
        return view('frontend.pages.login');
    }

    public function login(Request $request)
    {
        if (Auth::attempt(['email' => $request->email, 'password' => $request->password])) {
            $user = Auth::user();
            if ($request->ajax()) {
                return response()->json(['status' => 'success', 'message' => 'Logged in successfully.']);
            }
        }

        if ($request->ajax()) {
            return response()->json(['status' => 'error', 'message' => 'Invalid credentials.'], 401);
        }

        return response()->json(['status' => 'error', 'message' => 'Invalid credentials.'], 500);
    }

    public function logout(Request $request)
    {
        $request->session()->forget(['email', 'username']);

        Auth::logout();
        $request->session()->invalidate();
        $request->session()->regenerateToken();
        return redirect()->route('login')->with('message', 'Successfully logged out.');
    }

    public function sendResetLinkEmail(Request $request)
    {
        // Validate the email input
        $request->validate(['email' => 'required|email|exists:users,email']);

        // Attempt to send the password reset link via the Password facade
        $status = Password::sendResetLink(
            $request->only('email')
        );


        // Check the status of the reset link sending
        if ($status == Password::RESET_LINK_SENT) {
            return response()->json(['status' => 'success', 'message' => 'Verification link has been sent to your email.'], 200);
        } else {
            return response()->json(['status' => 'error', 'message' => 'Failed to send password reset link. Please try again.'], 500);
        }
    }

    public function showResetForm(Request $request, $token = null)
    {
        return view('auth.passwords.reset')->with(
            ['token' => $token, 'email' => $request->email]
        );
    }

    // Reset password
    public function reset(Request $request)
    {
        $request->validate([
            'email' => 'required|email|exists:users,email',
            'password' => 'required|confirmed|min:8',
            'token' => 'required'
        ]);

        $status = Password::reset(
            $request->only('email', 'password', 'password_confirmation', 'token'),
            function ($user, $password) {
                $user->password = bcrypt($password);
                $user->save();
            }
        );

        // If the reset was successful, redirect the user to login
        if ($status == Password::PASSWORD_RESET) {
            return redirect()->route('login')->with('status', 'Password has been reset successfully!');
        }

        // return back()->withErrors(['email' => 'Failed to reset password']);
        if ($status == Password::INVALID_TOKEN) {
            return back()->withErrors(['email' => 'This password reset token is invalid or has expired.']);
        } elseif ($status == Password::PASSWORD_RESET) {
            return redirect()->route('login')->with('status', 'Password has been reset successfully!');
        }        
    }

}
